@@ -17,6 +17,7 @@
 #include <gsmlib/gsm_nls.h>
 #include <gsmlib/gsm_me_ta.h>
 #include <gsmlib/gsm_parser.h>
+#include <gsmlib/gsm_sysdep.h>
 
 using namespace std;
 using namespace gsmlib;
@@ -30,8 +31,9 @@ Capabilities::Capabilities() :
   _omitsColon(true),            // FIXME
   _veryShortCOPSanswer(false),  // Falcom A2-1
   _wrongSMSStatusCode(false),   // Motorola Timeport 260
-  _CDSmeansCDSI(false)          // Nokia Cellular Card Phone RPE-1 GSM900 and
+  _CDSmeansCDSI(false),         // Nokia Cellular Card Phone RPE-1 GSM900 and
                                 // Nokia Card Phone RPM-1 GSM900/1800
+  _sendAck(false)               // send ack for directly routed SMS
 {
 }
 
@@ -83,6 +85,10 @@ void MeTa::init() throw(GsmException)
     _capabilities._CDSmeansCDSI = true;
   } 
 
+  // find out whether we are supposed to send an acknowledgment
+  Parser p(_at->chat("+CSMS?", "+CSMS:"));
+  _capabilities._sendAck = p.parseInt() >= 1;
+      
   // set GSM default character set
   try
   {
@@ -113,6 +119,17 @@ MeTa::MeTa(Ref<Port> port) throw(GsmException) : _port(port)
 //   init();
 // }
 
+void MeTa::setPIN(string pin) throw(GsmException)
+{
+  _at->chat("+CPIN=\"" + pin + "\"");
+}
+
+string MeTa::getPINStatus() throw(GsmException)
+{
+  Parser p(_at->chat("+CPIN?", "+CPIN:"));
+  return p.parseString();
+}
+
 void MeTa::setPhonebook(string phonebookName) throw(GsmException)
 {
   if (phonebookName != _lastPhonebookName)
@@ -154,6 +171,31 @@ string MeTa::setSMSStore(string smsStore, int storeTypes, bool needResultCode)
   return "";
 }
 
+void MeTa::getSMSStore(string &readDeleteStore,
+                       string &writeSendStore,
+                       string &receiveStore) throw(GsmException)
+{
+  Parser p(_at->chat("+CPMS?", "+CPMS:"));
+  writeSendStore = receiveStore = "";
+  readDeleteStore = p.parseString();
+  p.parseComma();
+  p.parseInt();
+  p.parseComma();
+  p.parseInt();
+  if (p.parseComma(true))
+  {
+    writeSendStore = p.parseString();
+    p.parseComma();
+    p.parseInt();
+    p.parseComma();
+    p.parseInt();
+    if (p.parseComma(true))
+    {
+      receiveStore = p.parseString();
+    }
+  }
+}
+
 void MeTa::waitEvent(GsmTime timeout) throw(GsmException)
 {
   if (_at->wait(timeout))
@@ -236,6 +278,17 @@ void MeTa::dial(string number) throw(GsmException)
   _at->chat("D" + number + ";");
 }
 
+void MeTa::answer() throw(GsmException)
+{
+  _at->chat("A");
+}
+
+void MeTa::hangup() throw(GsmException)
+{
+  _at->chat("H");
+
+}
+
 vector<OPInfo> MeTa::getAvailableOPInfo() throw(GsmException)
 {
   vector<OPInfo> result;
@@ -268,7 +321,11 @@ vector<OPInfo> MeTa::getAvailableOPInfo() throw(GsmException)
     //                    "(3,\"ORANGE\",\"ORANGE\",\"23433\")");
     //responses.push_back("(2,\"D1-TELEKOM\",,26201),"
     //                    "(3,\"D2  PRIVAT\",,26202),,(0,1,3,4),(0,2)");
-    
+    // some phones arbitrarily split the response into several lines
+    //responses.push_back("(1,\"AMENA\",,\"21403\"),"
+    //                    "(3,\"MOVISTAR\",,\"21407\"),");
+    //responses.push_back("(3,\"E VODAFONE\",,\"21401\"),,(0,1),(2)");
+
     // GSM modems might return
     // 1. quadruplets of info enclosed in brackets separated by comma
     // 2. several lines of quadruplets of info enclosed in brackets
@@ -277,6 +334,9 @@ vector<OPInfo> MeTa::getAvailableOPInfo() throw(GsmException)
     for (vector<string>::iterator i = responses.begin();
          i != responses.end(); ++i)
     {
+//       while (i->length() > 0 && ! isprint((*i)[i->length() - 1]))
+//         i->erase(i->length() - 1, 1);
+
       bool expectClosingBracket = false;
       Parser p(*i);
       while (1)
@@ -309,7 +369,7 @@ vector<OPInfo> MeTa::getAvailableOPInfo() throw(GsmException)
         result.push_back(opi);
         if (! p.parseComma(true)) break;
         // two commas ",," mean the list is finished
-        if (p.parseComma(true)) break;
+        if (p.getEol() == "" || p.parseComma(true)) break;
       }
       // without brackets, the ME/TA must use format 3.
       if (! expectClosingBracket) break;
@@ -733,6 +793,47 @@ int MeTa::getBatteryCharge() throw(GsmException)
   return p.parseInt();
 }
 
+int MeTa::getFunctionalityLevel() throw(GsmException)
+{
+  try {
+    Parser p(_at->chat("+CFUN?", "+CFUN:"));
+    // some phones return functionality level like "(2)"
+    bool expectClosingParen = p.parseChar('(', true);
+    int result = p.parseInt();
+    if (expectClosingParen)
+      p.parseChar(')');
+    return result;
+  }
+  catch (GsmException &x)
+  {
+    if (x.getErrorClass() == ChatError)
+    {
+      throw GsmException(_("Functionality Level commands not supported by ME"),
+			 MeTaCapabilityError);
+    } else {
+      throw;
+    }
+  }
+}
+
+void MeTa::setFunctionalityLevel(int level) throw(GsmException)
+{
+  try {
+    Parser p(_at->chat("+CFUN="  + intToStr(level)));
+  } catch (GsmException &x) {
+    if (x.getErrorClass() == ChatError)
+    {
+      // If the command AT+CFUN commands really aren't supported by the ME,
+      // then this will throw an appropriate exception for us.
+      getFunctionalityLevel();
+      // If the number was just out of range, we get here.
+      throw GsmException(_("Requested Functionality Level out of range"),
+			 ParameterError);
+    }
+    throw;
+  }
+}
+
 int MeTa::getSignalStrength() throw(GsmException)
 {
   Parser p(_at->chat("+CSQ", "+CSQ:"));
@@ -807,12 +908,78 @@ SMSStoreRef MeTa::getSMSStore(string storeName) throw(GsmException)
   return newSs;
 }
 
-void MeTa::sendSMS(SMSMessageRef smsMessage) throw(GsmException)
+void MeTa::sendSMS(Ref<SMSSubmitMessage> smsMessage) throw(GsmException)
 {
   smsMessage->setAt(_at);
   smsMessage->send();
 }
 
+void MeTa::sendSMSs(Ref<SMSSubmitMessage> smsTemplate, string text,
+                    bool oneSMS,
+                    int concatenatedMessageId)
+  throw(GsmException)
+{
+  assert(! smsTemplate.isnull());
+
+  // compute maximum text length for normal SMSs and concatenated SMSs
+  unsigned int maxTextLength, concMaxTextLength;
+  switch (smsTemplate->dataCodingScheme().getAlphabet())
+  {
+  case DCS_DEFAULT_ALPHABET:
+    maxTextLength = 160;
+    concMaxTextLength = 152;
+    break;
+  case DCS_EIGHT_BIT_ALPHABET:
+    maxTextLength = 140;
+    concMaxTextLength = 134;
+    break;
+  case DCS_SIXTEEN_BIT_ALPHABET:
+    maxTextLength = 70;
+    concMaxTextLength = 67;
+    break;
+  default:
+    throw GsmException(_("unsupported alphabet for SMS"),
+                       ParameterError);
+    break;
+  }
+
+  // simple case, only send one SMS
+  if (oneSMS || text.length() <= maxTextLength)
+  {
+    if (text.length() > maxTextLength)
+      throw GsmException(_("SMS text is larger than allowed"),
+                         ParameterError);
+    smsTemplate->setUserData(text);
+    sendSMS(smsTemplate);
+  }
+  else                          // send multiple SMSs
+  {
+    if (concatenatedMessageId != -1)
+      maxTextLength = concMaxTextLength;
+
+    int numMessages = (text.length() + maxTextLength - 1) / maxTextLength;
+    if (numMessages > 255)
+      throw GsmException(_("not more than 255 concatenated SMSs allowed"),
+                         ParameterError);
+    unsigned char numMessage = 0;
+    while (true)
+    {
+      if (concatenatedMessageId != -1)
+      {
+        unsigned char udhs[] = {0x00, 0x03, concatenatedMessageId,
+                                numMessages, ++numMessage};
+        UserDataHeader udh(string((char*)udhs, 5));
+        smsTemplate->setUserDataHeader(udh);
+      }
+      smsTemplate->setUserData(text.substr(0, maxTextLength));
+      sendSMS(smsTemplate);
+      if (text.length() < maxTextLength)
+        break;
+      text.erase(0, maxTextLength);
+    }
+  }
+}
+
 void MeTa::setMessageService(int serviceLevel) throw(GsmException)
 {
   string s;
@@ -1026,3 +1193,60 @@ void MeTa::setSMSRoutingToTA(bool enableSMS, bool enableCBS,
   _at->chat("+CNMI=" + chatString);
 }
 
+bool MeTa::getCallWaitingLockStatus(FacilityClass cl)
+  throw(GsmException)
+{
+  // some TA return always multiline response with all classes
+  // (Option FirstFone)
+  // !!! errors handling is correct (responses.empty() true) ?
+  vector<string> responses = 
+    _at->chatv("+CCWA=0,2," + intToStr((int)cl),"+CCWA:",true);
+  for (vector<string>::iterator i = responses.begin();
+       i != responses.end(); ++i)
+  {
+    Parser p(*i);
+    int enabled = p.parseInt();
+
+    // if the first time and there is no comma this 
+    // return direct state of classes
+    // else return all classes
+    if (i == responses.begin())
+    {
+      if (! p.parseComma(true))
+        return enabled == 1;
+    }
+    else
+      p.parseComma();
+
+    if (p.parseInt() == (int)cl)
+      return enabled == 1;
+  }
+  return false;
+
+}
+void MeTa::setCallWaitingLockStatus(FacilityClass cl, bool lock)
+  throw(GsmException)
+{
+  if(lock)
+    _at->chat("+CCWA=0,1," + intToStr((int)cl));
+  else
+    _at->chat("+CCWA=0,0," + intToStr((int)cl));
+}
+
+void MeTa::setCLIRPresentation(bool enable) throw(GsmException)
+{
+  if (enable)
+    _at->chat("+CLIR=1");
+  else
+    _at->chat("+CLIR=0");
+}
+
+int MeTa::getCLIRPresentation() throw(GsmException)
+{
+  // 0:according to the subscription of the CLIR service
+  // 1:CLIR invocation
+  // 2:CLIR suppression
+  Parser p(_at->chat("+CLIR?", "+CLIR:"));
+  return p.parseInt();
+}
+
